import { generateScriptableBanner } from '@scriptables/manifest/generateScriptableBanner';
import { existsSync } from 'fs';
import { readFile, writeFile } from 'fs/promises';
import { dirname, basename, resolve, posix, sep } from 'path';

// src/scriptable-banner.ts
var DEFAULT_MANIFEST_EXTENSIONS = [".manifest.json", ".manifest", ".json"];
function scriptableBanner(options = {}) {
  const {
    manifestExtensions = DEFAULT_MANIFEST_EXTENSIONS,
    warnOnMissingManifest = true,
    warnOnInvalidManifest = true,
    warnOnMissingEntry = true,
    getManifestPath
  } = options;
  return {
    name: "scriptable",
    setup(build) {
      let manifestCache = /* @__PURE__ */ new Map();
      if (!build.initialOptions.metafile) {
        console.warn(
          'Warning: "metafile" option was not enabled. Automatically setting "metafile: true" to allow scriptable plugin to modify output files.'
        );
        build.initialOptions.metafile = true;
      }
      build.onStart(() => {
        manifestCache.clear();
      });
      build.onResolve({ filter: /\.[tj]sx?$/ }, async (args) => {
        if (Array.isArray(build.initialOptions.entryPoints) && build.initialOptions.entryPoints.map(String).includes(args.path)) {
          const dir = dirname(args.path);
          const baseFileName = basename(args.path).replace(/\.[tj]sx?$/, "");
          const manifestPath = getManifestPath ? getManifestPath(args.path) : manifestExtensions.map((ext) => resolve(dir, `${baseFileName}${ext}`)).find(existsSync);
          if (manifestPath) {
            try {
              const manifestContent = await readFile(manifestPath, "utf8");
              const manifest = JSON.parse(manifestContent);
              const normalizedPath = posix.normalize(args.path.split(sep).join("/"));
              if (!manifest.name || !manifest.author || !manifest.version) {
                if (warnOnInvalidManifest) {
                  console.warn(`Invalid manifest file: ${manifestPath} - missing required fields`);
                }
                return null;
              }
              manifestCache.set(normalizedPath, manifest);
            } catch (error) {
              console.warn(`Failed to read manifest file: ${manifestPath}`, error);
            }
          } else if (warnOnMissingManifest) {
            console.warn(`No manifest file found for entry point: ${args.path}`);
          }
        }
        return null;
      });
      build.onEnd(async (result) => {
        if (!result.metafile || Object.keys(result.metafile.outputs).length === 0) {
          console.warn(
            'Skipping banner addition because "metafile" is not available or contains no outputs. Ensure "metafile: true" is set in esbuild options if write is enabled.'
          );
          return;
        }
        const outputs = result.metafile.outputs;
        for (const outputPath of Object.keys(outputs)) {
          if (build.initialOptions.write !== false && !existsSync(outputPath)) {
            console.warn(`Output file does not exist: ${outputPath}`);
            continue;
          }
          const outputMeta = outputs[outputPath];
          const entryPoint = outputMeta?.entryPoint ?? "";
          const normalizedEntryPath = entryPoint ? posix.normalize(entryPoint.split(sep).join("/")) : "";
          if (!normalizedEntryPath || !manifestCache.has(normalizedEntryPath)) {
            if (warnOnMissingEntry) {
              console.warn(`No matching entry point found for output file: ${outputPath}`);
            }
            continue;
          }
          try {
            const manifest = manifestCache.get(normalizedEntryPath) || {};
            const banner = generateScriptableBanner(manifest);
            if (result.outputFiles) {
              const outputFile = result.outputFiles.find((file) => file.path === outputPath || file.path === "<stdout>");
              if (outputFile) {
                outputFile.contents = Buffer.from(banner + outputFile.text.trimEnd());
              }
            } else {
              const content = await readFile(outputPath, "utf-8");
              const updatedContent = banner + content;
              await writeFile(outputPath, updatedContent, "utf8");
            }
          } catch (error) {
            console.error(`Failed to process output file: ${outputPath}`, error);
          }
        }
      });
    }
  };
}

export { scriptableBanner };
