import { findICloudDrivePaths, PathType } from 'findicloud';
import { copyFile } from 'fs/promises';
import { basename, join } from 'path';

// src/scriptable-deploy.ts

// src/utils.ts
function createFileFilter(options) {
  const { include = [], exclude = [] } = options || {};
  const includePatterns = Array.isArray(include) ? include : [include];
  const excludePatterns = Array.isArray(exclude) ? exclude : [exclude];
  return function filter(filename) {
    if (includePatterns.length === 0 && excludePatterns.length === 0) {
      return true;
    }
    for (const pattern of excludePatterns) {
      if (matchPattern(filename, pattern)) {
        return false;
      }
    }
    if (includePatterns.length === 0) {
      return true;
    }
    for (const pattern of includePatterns) {
      if (matchPattern(filename, pattern)) {
        return true;
      }
    }
    return false;
  };
}
function matchPattern(filename, pattern) {
  if (pattern instanceof RegExp) {
    return pattern.test(filename);
  }
  if (typeof pattern === "string") {
    return filename.includes(pattern);
  }
  if (typeof pattern === "function") {
    return pattern(filename);
  }
  return false;
}

// src/scriptable-deploy.ts
var findICloudBugsUrl = "https://github.com/hexxspark/findicloud/issues";
function scriptableDeploy(options = {}) {
  const { verbose = false, continueOnError = true, scriptableDir, filter } = options;
  const fileFilter = createFileFilter(filter);
  let targetDir;
  return {
    name: "scriptable-deploy",
    setup(build) {
      if (!build.initialOptions.metafile) {
        build.initialOptions.metafile = true;
      }
      build.onStart(async () => {
        try {
          if (scriptableDir) {
            targetDir = scriptableDir;
          } else {
            const [found] = await findICloudDrivePaths({ appNamePattern: "Scriptable", types: [PathType.APP_STORAGE] });
            targetDir = found?.path;
          }
          if (!targetDir) {
            throw new Error(
              `Scriptable directory not found in iCloud Drive. Please ensure iCloud Drive is installed and Scriptable app is synced with iCloud. If iCloud Drive is not installed, you can download it from: https://support.apple.com/en-us/HT204283. If the issue persists, please report it at: ${findICloudBugsUrl}`
            );
          }
          if (verbose) {
            console.log(`[scriptable-deploy] Target directory: ${targetDir}`);
          }
        } catch (error) {
          const message = `Failed to find Scriptable directory: ${error.message}`;
          if (!continueOnError) {
            throw new Error(message);
          }
          console.error(`[scriptable-deploy] ${message}`);
        }
      });
      build.onEnd(async (result) => {
        if (!result.metafile || !targetDir) {
          return;
        }
        try {
          const outputs = Object.keys(result.metafile.outputs).filter((file) => fileFilter(basename(file)));
          for (const file of outputs) {
            const fileName = basename(file);
            const targetPath = join(targetDir, fileName);
            await copyFile(file, targetPath);
            if (verbose) {
              console.log(`[scriptable-deploy] Deployed: ${file} -> ${targetPath}`);
            }
          }
          if (verbose) {
            console.log(`[scriptable-deploy] Successfully deployed ${outputs.length} files`);
          }
        } catch (error) {
          const message = `Deploy failed: ${error.message}`;
          if (!continueOnError) {
            throw new Error(message);
          }
          console.error(`[scriptable-deploy] ${message}`);
        }
      });
    }
  };
}

export { scriptableDeploy };
