import type {Plugin} from 'esbuild';
import {findICloudDrivePaths, PathType} from 'findicloud';
import {copyFile} from 'fs/promises';
import {basename, join} from 'path';

import {FileFilterOptions} from './types';
import {createFileFilter} from './utils';

export interface ScriptableDeployOptions {
  /**
   * Whether to show detailed logs during deployment
   * Default: false
   */
  verbose?: boolean;

  /**
   * Whether to continue building when deployment fails
   * Default: true
   */
  continueOnError?: boolean;

  /**
   * Custom Scriptable directory path
   * If not specified, will automatically search in iCloud Drive
   */
  scriptableDir?: string;

  /**
   * Custom file filter function
   * @param fileName Output file name
   * @returns Whether to deploy this file
   */
  filter?: FileFilterOptions;
}

const findICloudBugsUrl = 'https://github.com/hexxspark/findicloud/issues';

export function scriptableDeploy(options: ScriptableDeployOptions = {}): Plugin {
  const {verbose = false, continueOnError = true, scriptableDir, filter} = options;
  const fileFilter = createFileFilter(filter);

  let targetDir: string;

  return {
    name: 'scriptable-deploy',
    setup(build) {
      // Ensure metafile is enabled
      if (!build.initialOptions.metafile) {
        build.initialOptions.metafile = true;
      }

      build.onStart(async () => {
        try {
          if (scriptableDir) {
            targetDir = scriptableDir;
          } else {
            // Find iCloud paths
            const [found] = await findICloudDrivePaths({appNamePattern: 'Scriptable', types: [PathType.APP_STORAGE]});
            targetDir = found?.path;
          }

          if (!targetDir) {
            throw new Error(
              'Scriptable directory not found in iCloud Drive. ' +
                'Please ensure iCloud Drive is installed and Scriptable app is synced with iCloud. ' +
                'If iCloud Drive is not installed, you can download it from: https://support.apple.com/en-us/HT204283. ' +
                `If the issue persists, please report it at: ${findICloudBugsUrl}`,
            );
          }

          if (verbose) {
            console.log(`[scriptable-deploy] Target directory: ${targetDir}`);
          }
        } catch (error) {
          const message = `Failed to find Scriptable directory: ${(error as Error).message}`;
          if (!continueOnError) {
            throw new Error(message);
          }
          console.error(`[scriptable-deploy] ${message}`);
        }
      });

      build.onEnd(async result => {
        if (!result.metafile || !targetDir) {
          return;
        }

        try {
          const outputs = Object.keys(result.metafile.outputs).filter(file => fileFilter(basename(file)));

          for (const file of outputs) {
            const fileName = basename(file);
            const targetPath = join(targetDir, fileName);

            await copyFile(file, targetPath);

            if (verbose) {
              console.log(`[scriptable-deploy] Deployed: ${file} -> ${targetPath}`);
            }
          }

          if (verbose) {
            console.log(`[scriptable-deploy] Successfully deployed ${outputs.length} files`);
          }
        } catch (error) {
          const message = `Deploy failed: ${(error as Error).message}`;
          if (!continueOnError) {
            throw new Error(message);
          }
          console.error(`[scriptable-deploy] ${message}`);
        }
      });
    },
  };
}
