import {FileFilterOptions, FilterPattern} from './types';

/**
 * Creates a file filter function based on provided options
 * @param options Filter configuration options
 * @returns Function that determines whether a file should be included
 */
export function createFileFilter(options?: FileFilterOptions) {
  const {include = [], exclude = []} = options || {};

  // Convert to array
  const includePatterns = Array.isArray(include) ? include : [include];
  const excludePatterns = Array.isArray(exclude) ? exclude : [exclude];

  return function filter(filename: string): boolean {
    // If no filter conditions, return true
    if (includePatterns.length === 0 && excludePatterns.length === 0) {
      return true;
    }

    // First check exclude patterns
    for (const pattern of excludePatterns) {
      if (matchPattern(filename, pattern)) {
        return false; // If matches any exclude pattern, return false directly
      }
    }

    // If no include patterns and passed exclude check, return true
    if (includePatterns.length === 0) {
      return true;
    }

    // Check include patterns
    for (const pattern of includePatterns) {
      if (matchPattern(filename, pattern)) {
        return true;
      }
    }

    return false;
  };
}

/**
 * Match if filename matches the specified pattern
 */
function matchPattern(filename: string, pattern: FilterPattern): boolean {
  if (pattern instanceof RegExp) {
    return pattern.test(filename);
  }
  if (typeof pattern === 'string') {
    return filename.includes(pattern);
  }
  if (typeof pattern === 'function') {
    return pattern(filename);
  }
  return false;
}
